@@ -1,6 +1,6 @@
 package Gtk2::Ex::HyperTextBuffer;
 
-our $VERSION = '0.12';
+our $VERSION = '0.16';
 
 use strict;
 use Gtk2;
@@ -54,6 +54,7 @@ our @_targets = (
 	{ target => 'TEXT'          },
 	{ target => 'COMPOUND_TEXT' },
 	{ target => 'UTF8_STRING'   },
+	{ target => 'text/plain'    },
 	{ target => 'ZIM_TREE'      },
 );
 
@@ -71,6 +72,13 @@ sub INIT_INSTANCE {
 	my $self = shift;
 	$self->{edit_mode_tags} = [];
 	$self->{image_marks} = [];
+#	$self->signal_connect(mark_set => sub {
+#			my ($self, $iter, $mark) = @_;
+#			return unless $mark->get_name eq 'insert';
+#			my @names = map {$_->get_property('name')} $self->_get_open_tags($iter);
+#			warn "Tags: @names\n";
+#			#$self->set_edit_mode_from_cursor;
+#		} );
 }
 
 =item C<clear()>
@@ -155,6 +163,7 @@ sub insert_blocks_at_cursor {
 	# Make sure not to modify content of the reference !
 	# TODO: check default editable / region editable
 	my ($self, @blocks) = @_;
+	#use Data::Dumper; warn Dumper \@blocks;
 #	$self->set_edit_mode_from_cursor
 #		unless defined $self->{edit_mode_tags}; # FIXME
 	$self->{edit_mode_tags} ||= [];
@@ -177,7 +186,7 @@ sub insert_blocks_at_cursor {
 			}
 			elsif ($type eq 'link') {
 				my $bit = (@$_ == 3 and $$_[2] eq $$meta{to});
-				$tag = $self->create_link_tag([$bit, $$meta{to}]);
+				$tag = $self->create_link_tag($bit ? undef : $$meta{to});
 			}
 			else { $tag = $self->get_tag_table->lookup($type) }
 			
@@ -216,16 +225,22 @@ Insert an image in the buffer at the cursor.
 sub insert_image_from_file {
 	my ($self, $src) = @_;
 	my $file = $self->{base} ? File::Spec->rel2abs($src, $self->{base}) : $src;
-	#print "file: $$meta{src}, base: $self->{base} => $file\n";
+	#warn "file: $src, base: $self->{base} => $file\n";
 
-	my $image = Gtk2::Gdk::Pixbuf->new_from_file($file) if -f $file and -r _;
-	$image = Gtk2::Image->new->render_icon('gtk-missing-image', 'button' )
-		unless $image;
-	die "could not insert image: $file\n" unless $image;
-
-	$image->{image_src} = $src;
+	my $pixbuf = undef;
+	eval { $pixbuf = Gtk2::Gdk::Pixbuf->new_from_file($file) }
+		if (-f $file and -r _) ;
+	$pixbuf ||= Gtk2::Image->new->render_icon('gtk-missing-image', 'dialog')->copy;
+	die "Could not insert image: $file\n" unless $pixbuf;
+	
+	$pixbuf->{image_src} = $src;
+	$pixbuf->{image_file} = $file;
 	my $iter = $self->get_iter_at_mark( $self->get_insert );
-	$self->insert_pixbuf($iter, $image);
+
+	#my $mark = $self->create_mark(undef, $iter, 0);
+	#$mark->set_visible(1);
+	
+	$self->insert_pixbuf($iter, $pixbuf);
 
 	return 1;
 }
@@ -248,9 +263,11 @@ sub get_parse_tree {
 	my @stack = ($root);
 	my $iter = $start->copy;
 	for ($iter->get_tags) {
+		my $link_data = Gtk2::Ex::HyperTextView->get_link_at_iter($iter, $_)
+			if $_->{is_link}; # FIXME quick hack here
 		my $tag = $_->{is_link}
-			? [ 'link', {to => $_->{link_data}[1]} ]
-			: [ $_->get_property('name'), {}       ] ;
+			? [ 'link', {to => $link_data}   ]
+			: [ $_->get_property('name'), {} ] ;
 		next if $$tag[0] =~ /gtkspell/;
 		push @{$stack[-1]}, $tag;
 		push @stack, $tag;
@@ -285,9 +302,11 @@ sub get_parse_tree {
 			pop @stack;
 		}
 		for ($iter->get_toggled_tags(1)) { # opened tags
+			my $link_data = Gtk2::Ex::HyperTextView->get_link_at_iter($iter, $_)
+				if $_->{is_link}; # FIXME quick hack here
 			my $tag = $_->{is_link}
-				? [ 'link', {to => $_->{link_data}[1]} ]
-				: [ $_->get_property('name'), {}       ] ;
+				? [ 'link', {to => $link_data}   ]
+				: [ $_->get_property('name'), {} ] ;
 			next if $$tag[0] =~ /gtkspell/;
 			push @{$stack[-1]}, $tag;
 			push @stack, $tag;
@@ -347,11 +366,6 @@ sub get_parse_tree {
 
 Returns an anonymous L<Gtk2::TextIter> that holds LINK_DATA.
 
-LINK_DATA should be an array reference. The first item in this array
-should be a boolean descibing whether the link matches the text it is applied
-to. If this boolean is set to tru the second element in the array will be update
-to hold the text if the buffer is edited.
-
 Every link tag should only be applied to one text region.
 
 =cut
@@ -531,11 +545,23 @@ TODO
 
 =cut
 
+sub _get_open_tags {
+	# iter->get_tags has "left" gravity
+	# this function has "right" gravity
+	my ($self, $iter) = @_;
+	my @tags = $iter->get_tags;
+	for my $t ($iter->get_toggled_tags(1)) { # tags that are toggles on here
+		@tags = grep {$_ ne $t} @tags;
+	}
+	push @tags, $iter->get_toggled_tags(0);
+	return @tags;
+}
+
 sub set_edit_mode_from_cursor {
 	my $self = shift;
 	my $iter = $self->get_iter_at_mark( $self->get_insert );
 	$self->{edit_mode_tags} = [$iter->get_tags];
-	#print join ' ', "Mode: ", map(
+	#print join ' ', "Mode:", map(
 	#	{$_->get_property('name')} @{$self->{edit_mode_tags}}
 	#), "\n";
 }
