@@ -3,7 +3,7 @@ package Zim::History;
 use strict;
 use Storable qw/nstore retrieve/;
 
-our $VERSION = '0.12';
+our $VERSION = '0.16';
 
 =head1 NAME
 
@@ -44,6 +44,12 @@ Third argument is the maximum number of items in the history.
 
 =cut
 
+# We use two stacks here called "hist" and "recent"
+# both contain the same kind of records (hashes with state data for each page)
+# To keep everything in sync we want only one record per page
+# which is linked by reference one or more times.
+
+
 sub new {
 	my ($class, $file, $view, $max) = @_;
 	my $self = bless {
@@ -85,47 +91,68 @@ sub write {
 	nstore([$VERSION, @{$self}{qw/point hist recent/}], $self->{file});
 }
 
-=item C<set_current(PAGE)>
+=item C<set_current(PAGE, FROM_HIST)>
 
 Give the page object that is going to be displayed in the 
 PageView. Set a new current before updating the PageView
 so the History object has time to save the state of the PageView
 if necessary.
 
+The FROM_HIST boolean signifies whether this page was loaded
+from (this) history and for example as a result of following
+a link. The behavior is subtily different when this boolean
+is true.
+
 =cut
 
 sub set_current {
-	my ($self, $page) = @_;
+	my ($self, $page, $from_hist) = @_;
 
-	my $current = $self->get_current;
+	my $prev = $self->get_current; # do not use get_current below !
 	my $name = $page->name;
-	$self->{page} = $page; # set _after_ calling get_current()
-	return if $current and $name eq $current->{name}; # directly redundant
-
+	$self->{page} = $page;
+	return $self->_set_recent($prev, $from_hist)
+		if $prev and $name eq $prev->{name}; # directly redundant
+	
+	# lookup record, or create new record
+	my $rec = $self->_get_record($name) || {
+		name => $name,
+		namespace => $page->namespace,
+		basename => $page->basename,
+	};
+	
 	# update hist stack
 	my $point = $self->{point};
 	my $hist = $self->{hist};
 	if    ($point >= $self->{max}) { shift @$hist }
 	elsif (defined $$hist[$point]) { $point++     }
 	splice @$hist, $point; # clear forw stack
-	$$hist[$point] = undef;
+	$$hist[$point] = $rec;
 	$self->{point} = $point;
-	# note: do not call get_current below this line
 
-	# update recent stack
+	$self->_set_recent($rec, $from_hist);
+}
+
+sub _set_recent {
+	# update recent stack to reflect the current page
+	# stuff may have been deleted (fro example because the page was deleted)
+	# while the page still is in the history, so check all possibilities
+	my ($self, $rec, $from_hist) = @_;
 	my $recent = $self->{recent};
-	for (@$recent) { return if $_->{name} eq $name } # check redundancy
-	my $rec = { map {$_ => $page->$_} qw/name namespace basename/ };
-	$$hist[$point] = $rec; # force linking of ref
+	my $name = $rec->{name};
+	if ($from_hist) {
+		return if grep {$_->{name} eq $name} @$recent; # return if exists
+	}
+	else {
+		@$recent = grep {$_->{name} ne $name} @$recent; # remove redundancy
+	}
 	shift @$recent if $#$recent >= $self->{max};
 	push @$recent, $rec;
-
-	#use Data::Dumper; print 'hist, recent: ', Dumper($hist, $recent);
 }
 
 =item C<get_current()>
 
-Returns the curernt history object. When possible asks the PageView
+Returns the current history object. When possible asks the PageView
 objects for the current state information.
 
 =cut
@@ -141,7 +168,9 @@ sub get_current {
 		my $name = $page->name;
 		$rec = $self->_get_record($name) || {
 			name => $name,
-			map {$_ => $page->$_} qw/namespace basename/  };
+			namespace => $page->namespace,
+			basename => $page->basename,
+		};
 		$self->{hist}[ $self->{point} ] = $rec;
 		#use Data::Dumper; print "rec: ", Dumper $rec;
 	}
@@ -151,6 +180,18 @@ sub get_current {
 	return $rec;
 }
 
+=item C<delete_recent(PAGE)>
+
+Deletes the entry for PAGE from the recent pages stack.
+
+=cut
+
+sub delete_recent {
+	my ($self, $name) = @_;
+	#warn "Deleting $name from recent\n";
+	@{$self->{recent}} = grep {$_->{name} ne $name} @{$self->{recent}};
+}
+
 =item C<back(INT)>
 
 Go back one or more steps in the history stack.
@@ -206,11 +247,10 @@ The integer is the index of the current page in this stack.
 
 sub get_recent {
 	my $self = shift;
-	my $name = $self->get_current->{name};
+	my $ref = $self->get_current;
 	my $i = 0;
 	for (@{$self->{recent}}) {
-		#print "Comparing $name with $_->{name}\n";
-		last if $_->{name} eq $name;
+		last if $_ eq $ref;
 		$i++;
 	}
 	return $i, @{$self->{recent}};
