@@ -2,7 +2,7 @@ package Zim::Page;
 
 use strict;
 
-our $VERSION = '0.13';
+our $VERSION = '0.16';
 
 use overload
 	'""' => sub { $_[0]->{name} }, # \&name results in bugs ?
@@ -119,6 +119,10 @@ sub basename {
 
 Returns the namespace to which this page belongs.
 
+=item C<namespaces()>
+
+Like C<namespace()> but returns the namespace path as a list.
+
 =cut
 
 sub namespace {
@@ -131,6 +135,13 @@ sub namespace {
 	return $name;
 }
 
+sub namespaces {
+	my $self = shift;
+	my $name = $self->namespace(@_);
+	$name =~ s/^:+|:+$//g;
+	return split /:+/, $name;
+}
+
 =item C<status(STRING)>
 
 Set or get a status string for this page.
@@ -169,9 +180,54 @@ sub move { $_[0]->{repository}->root->move_page(@_) }
 
 sub delete { $_[0]->{repository}->delete_page(@_) }
 
-=item C<resolve_link(LINK)>
+=item C<parse_link(LINK)>
 
-Returns a page object for LINK where LINK can be resolved
+Returns the link type and link target for LINK. The type can be 'page',
+'file', 'mail' or 'man' - for urls the protocol is used as type.
+
+The target returned is usually the same as LINK but can be different
+for example for interwiki links.
+
+=cut
+
+sub parse_link {
+	my ($self, $link) = @_;
+	if ($link =~ s/^(\w[\w\+\-\.]+)\?//) { # interwiki link
+		my $key = $1;
+		my $l = $self->{repository}->root->interwiki_lookup($key, $link);
+		if (defined $l) { $link = $l }
+		else {
+			return 'man', $link if $key eq 'man'; # man?zim should work
+			return undef;
+		}
+	}
+	$_ = $link;
+	
+	my $type;
+	if    (m#^(\w+[\w\+\-\.]+)://#) { # urls
+		$type = $1;
+		$type = 'file' if $^O eq 'MSWin32' and $type eq 'file';
+	}
+	elsif (m#^mailto:|^\S+\@\S+\.\w+$#) { # email
+		$link =~ s#^(mailto:)?#mailto:#;
+		$type = 'mail';
+	}
+	else {
+		$type =
+			m#/#		? 'file' :
+			m#:#		? 'page' :
+			m#\(\d\w*\)$#	? 'man'  : 'page' ;
+	}
+
+	$link = Zim::File->abs_path($link, $self->{properties}{base})
+		if $type eq 'file';
+
+	return $type, $link;
+}
+
+=item C<resolve_link(NAME)>
+
+Returns a page object for NAME where NAME can be resolved
 relative to the current page.
 
 =cut
@@ -181,6 +237,19 @@ sub resolve_link { # 'cloning' hack needed when exporting
 	$obj->{repository}->resolve_link(@_)
 }
 
+=item C<match_word(WORD)>
+
+TODO: stable api for this
+
+=cut
+
+sub match_word { # TODO optimize by caching found links
+	my ($self, $word) = @_;
+	return $self->{repository}->can('_match_word')
+		? $self->{repository}->_match_word($self, $word)
+		: undef ;
+}
+
 =item C<clone(SOURCE)>
 
 Import content from object SOURCE into this object.
