@@ -2,8 +2,9 @@ package Zim::Page::Text;
 
 use strict;
 use Zim::Page;
+#use Fcntl ':flock'; # import LOCK_* constants
 
-our $VERSION = '0.12';
+our $VERSION = '0.16';
 
 our @ISA = qw/Zim::Page/;
 
@@ -105,13 +106,14 @@ sub set_source {
 
 =item C<get_source(MODE)>
 
-Returns an IO object or undef if there is none.
+Returns an IO object or undef when there is none.
+This method dies when it fails opening a given source.
 In general pages that have status "new" will not yet have a source.
 
 MODE is optional and can either be 'r' or 'w' depending on whether you
 would like the source to be opened for reading or for writing.
 
-Do not forget to close the IO object when you are done with it.
+Do not forget to close the IO object when you are done with it !
 
 ( When using source the repository should also support the
 C<get_source(SOURCE, MODE)> method. )
@@ -195,17 +197,91 @@ to the IO object.
 sub set_parse_tree {
 	my ($self, $tree) = @_;
 	$self->{status} = ''; # remove "new" or "deleted"
+	$self->{_links} = [ $self->_list_links($tree) ];
 	if (defined $self->{source}) {
 		my $io = $self->get_source('w')
 			|| die "Could not save parse tree, did not get an IO object.\n";
+		#flock($io, LOCK_EX); # lock # Move function to IO::File::Zim
 		$self->{format}->save_tree($io, $tree, $self);
+		#flock($io, LOCK_UN); # unlock
 		$io->close;
+		
+		$self->{repository}->_cache_page($self)
+			if $self->{repository}->can('_cache_page');
 	}
 	else {
 		$self->{parse_tree} = $tree;
 	}
 }
 
+sub _list_links {
+	my $self = shift;
+	my $node = shift;
+	unless ($node) {
+		return @{$self->{_links}} if $self->{_links};
+		$node = $self->get_parse_tree;
+	}
+
+	my @list;
+	for (2 .. $#$node) {
+		my $n = $$node[$_];
+		next unless ref $n;
+		if ($$n[0] eq 'link') {
+			my $src = $$n[1]{to};
+			my $name = join '', @$n[2..$#$n];
+			push @list, [$name, $src];
+		}
+		else { push @list, _list_links($self, $n) } # recurse
+	}
+
+	return grep defined($_), @list;
+}
+
+sub _list_backlinks {
+	return $_[0]->{repository}->can('_list_backlinks')
+		? $_[0]->{repository}->_list_backlinks($_[0])
+		: () ;
+}
+
+sub _update_links {
+	# FIXME ugly resolve lookup to be sure bout links
+	# should implement a "compare" function for page names
+	# end use the old object to compare links against
+	my ($self, $type, $old, $new) = @_;
+	return unless $self->{format};
+	warn "Updating links in ", $self->name, "\n";
+	my $tree = $self->get_parse_tree;
+	my $done = 0;
+	for my $ref (_extract('link', $tree)) {
+		my ($t, $l) = $self->parse_link($$ref[1]{to});
+		next unless $t eq $type;
+		$l = $self->resolve_link($l) || next;
+		$l = $l->name;
+		warn "Matching $l to $old\n";
+		next unless $l eq $old;
+		$old = $$ref[1]{to};
+		warn "Updating $old => $new\n";
+		$$ref[1]{to} = $new->name;
+		$$ref[2] = $$ref[1]{to} if $$ref[2] eq $old;
+		$done++;
+	}
+	use Data::Dumper; warn Dumper $tree;
+	$self->set_parse_tree($tree) if $done;
+	warn "Updated $done links\n";
+}
+
+sub _extract {
+	my ($type, $tree) = @_;
+	my @nodes;
+	for (2 .. $#$tree) {
+		my $node = $$tree[$_];
+		next unless ref $node;
+		push @nodes, ($$node[0] eq $type)
+			? $node : _extract($type, $node) ; # recurs
+	}
+	return grep defined($_), @nodes;
+}
+
 1;
 
 __END__
