@@ -1,14 +1,15 @@
 package Zim::Repository;
 
+use utf8;
 use strict;
 use Carp;
 use UNIVERSAL qw/isa/;
 use File::Spec;
+use File::BaseDir qw/xdg_config_files xdg_data_files/;
+use Zim::Selection;
 use Zim::File;
 
-our $VERSION = '0.13';
-
-$Carp::CarpLevel = 1;
+our $VERSION = '0.16';
 
 =head1 NAME
 
@@ -50,36 +51,40 @@ a onfig file.
 =cut
 
 sub new {
-	my ($class, $parent, $namespace, $dir, $format) = @_;
+	my ($class, $parent, $namespace, $node, $format) = @_;
 		# passing along $format here is a hack ! FIXME
 	$namespace =~ s/:?$/:/;
-	$dir = File::Spec->rel2abs($dir);
+	$node = Zim::File->abs_path($node);
+	#warn "repository root: $node\n";
+	die "Not a directory: $node\n" if -e $node and ! -d $node;
+	
 	my $self = bless {
 		parent => $parent,
 		namespace => $namespace,
-		dir => $dir,
+		dir => $node,
 		config => {},
 	}, $class;
 	
-	return $self unless length $dir;
+	return $self unless length $node;
 	
 	# load config file
-	my $conf = Zim::File->new($dir, '.zim.config');
-	for ($conf->read) {
-		chomp;
-		next unless /\S/;
-		my ($opt, $val) = split '=', $_, 2;
-		if ($opt =~ /^:/) { # opt is namespace
-			my ($class, $arg) = split '=', $val, 2;
-			my @arg = split ',', $arg if defined $arg;
-			$self->add_child($opt, $class, @arg);
+	#if ($type eq 'dir') {
+		my $conf = Zim::File->new($node, '.zim.config');
+		for ($conf->read) {
+			chomp;
+			next unless /\S/;
+			my ($opt, $val) = split '=', $_, 2;
+			if ($opt =~ /^:/) { # opt is namespace
+				my ($class, $arg) = split '=', $val, 2;
+				my @arg = defined($arg) ? (split ',', $arg) : ();
+				$self->add_child($opt, $class, @arg);
+			}
+			else { $self->{config}{$opt} = $val }
 		}
-		else { $self->{config}{$opt} = $val }
-	}
+	#}
 	
 	# default config
-	$self->add_child(':', 'Files', $dir, $format)
-		unless defined $self->{':'};
+	$self->add_child(':', 'Files', $node, $format) unless defined $self->{':'};
 
 	return $self;
 }
@@ -92,6 +97,17 @@ Returns a hash with config options.
 
 sub config { $_[0]->{config} }
 
+=item C<base()>
+
+Returns the base directory for files.
+
+The "base" directories for individual pages should either be the same as
+this directory or a subdirectory of it.
+
+=cut
+
+sub base { return $_[0]->{dir} }
+
 ## 3 methods for checking arguments ##
 
 sub _check_page {
@@ -103,14 +119,19 @@ sub _check_page {
 	else { goto \&_check_page_name }
 }
 
+# Valid page names can contain letters, numbers and ".", "-", "(" and ")"
+# The first character of a name can only be a letter or a number
+# Absolute names start with a ":"
+# Namespaces end in a ":"
+
 sub _check_page_name {
 	croak "\"$_[0]\" is not a valid page name"
-		unless $_[0] =~ /^:/ and $_[0] =~ /[^:]/;
+		unless $_[0] =~ /^(?::+\w[\w\.\-\(\)]*)+$/;
 }
 
 sub _check_namespace {
 	croak "\"$_[0]\" is not a valid namespace"
-		unless $_[0] =~ /^:/ and $_[0] =~ /:$/;
+		unless $_[0] =~ /^(?::+|(?::+\w[\w\.\-\(\)]*)+:+)$/;
 }
 
 =item C<list_pages(NAMESPACE)>
@@ -157,12 +178,34 @@ so NAME is sanitized first.
 
 sub resolve_page {
 	my ($self, $name) = @_;
-	$name =~ s/^:?/:/; # simple fixup
+	#print STDERR "resolved $name to ";
+	$name =~ s/^:?/:/;			# make absolute
+	$name =~ s/:+$//;			# not a namespace
+	$name =~ s/[^:\w\.\-\(\)]/_/g;		# replace forbidden chars
+	$name =~ s/(:+)[\_\.\-\(\)]+/$1/g;	# remove non-letter at begin
+	$name =~ s/_+(:|$)/$1/g;		# remove trailing underscore
+	#print STDERR "$name\n";
 	$name =~ $self->{regex};
 	#warn "Dispatching resolve_page('$name') to ':$1'\n";
 	return $self->{":$1"}->resolve_page($name);
 }
 
+=item C<resolve_namespace(NAME)>
+
+Returns a namespace string. Used to sanitize user input.
+
+=cut
+
+sub resolve_namespace {
+	my ($self, $name) = @_;
+	$name =~ s/^:?/:/;			# make absolute
+	$name =~ s/:?$/:/;			# is a namespace
+	$name =~ s/[^:\w\.\-\(\)]/_/g;		# replace forbidden chars
+	$name =~ s/(:+)[\_\.\-\(\)]+/$1/g;	# remove non-letter at begin
+	$name =~ s/_+(:|$)/$1/g;		# remove trailing underscore
+	return $name;
+}
+
 =item copy_page(SOURCE, TARGET)
 
 Copy page SOURCE to TARGET. Both arguments can be either page names
@@ -217,6 +260,23 @@ sub delete_page {
 	return $page;
 }
 
+=item C<search(PAGE, QUERY)>
+
+TODO stable api for this
+
+=cut
+
+# We could probably use some kind of "query" object that coordinates searches
+# on subsets of pages  and combines backlink lists with other keywords
+
+sub search {
+	my ($self, $query) = @_;
+	return map {
+		$self->{$_}->can('_search')
+			? ( $self->{$_}->_search($query) ) : ()
+	} grep /^:/, keys %$self;
+}
+
 =item C<export(PAGE, %OPTS)>
 
 TODO stable api for this
@@ -227,8 +287,9 @@ TODO stable api for this
 
 sub export {
 	my ($self, $page, %opts) = @_;
+	$page = shift @$page; # TODO
 	for (qw/format template root/) {
-		die "for exporting you need at least to provide a format, a template and an output root dir.\n" unless length $opts{$_};
+		die "for exporting you need to provide at least a format, a template and an output root dir.\n" unless length $opts{$_};
 	}
 
 	$page = $page->name if ref $page;
@@ -239,50 +300,65 @@ sub export {
 	#print STDERR "export template: $opts{template}\n";
 	$exporter->{_template} = $opts{template}; # FIXME better interface for this - do not confuse with the template for new pages
 	
-	my $r = defined($opts{recurs}) ? $opts{recurs} : 1 ;
+	$opts{recurs} = 1 unless defined $opts{recurs};
 	if (length $namespace) {
-		$self->_export_namespace($exporter, $page, $r);
+		my $index = $self->_export_namespace($exporter, $page, 0, \%opts);
+		#use Data::Dumper; print Dumper $index;
+		$self->_write_index($exporter, $index);
 	}
 	else {
-		$self->_export_page($exporter, $page);
+		$self->_export_page($exporter, $page, \%opts);
 	}
 
 }
 
-
 sub _export_namespace {
-	my ($self, $exporter, $namespace, $recurs) = @_;
+	my ($self, $exporter, $namespace, $lvl, $opts) = @_;
 	_check_namespace($namespace);
 	
-	my $index = 1;
-	my @pages;
-	for my $page ($self->list_pages($namespace)) {
-		$index = 0 if $page =~ /^index/i;
-		$self->_export_namespace($exporter, $namespace.$page, $recurs)
-			if $recurs and $page =~ /:$/; # recurs
-		push @pages, $page
-			if $self->_export_page($exporter, $namespace.$page);
+	my @index = $self->list_pages($namespace);
+	my @pages = grep $$_[1]->exists,
+	            map [$_ => $self->get_page($namespace.$_)], @index;
+	
+	for (0 .. $#pages) {
+		my $page = $pages[$_][1];
+		$exporter->{_next} = $pages[($_+1)%@pages][1];
+		$exporter->{_prev} = $pages[$_-1][1];
+		$self->_export_page($exporter, $page, $opts);
+	}
+	my %pages = map @$_, @pages;
+	@pages = (); # free
+	
+	my %index;
+	if ($$opts{recurs}) {
+		$index{$_} = $self->_export_namespace(
+			$exporter, $namespace.$_, $lvl+1, $opts )
+			for grep /:$/, @index;
 	}
 
-	if ($index && @pages) { # FIXME option for this
-		print STDERR "Writing index for $namespace\n";
-		#print STDERR join(', ', @pages), "\n";
-		$index = $exporter->get_page($namespace.'index');
-		$index->set_parse_tree( ['Document', {},
-			['head1', {}, "Document index for $namespace"],
-			['Para', {},
-				map {("* ", ['link', {to => $_}, $_], "\n")} @pages
-			]
-		] );
+	my $index = [$lvl, $namespace];
+	for my $i (@index) {
+		my $p = $i;
+		$p =~ s/:*$//;
+		push @$index, $p         if $pages{$i};
+		push @$index, $index{$i} if $index{$i};
 	}
-	
+	return $index;
 }
 
 sub _export_page {
-	my ($self, $exporter, $page) = @_;
-	_check_page_name($page);
+	my ($self, $exporter, $page, $opts) = @_;
+	
+	$page =~ s/:+$// unless ref $page; # TODO doesn't belong here
+	
+	my $source;
+	if (ref $page) { ($source, $page) = ($page, $page->name) }
+	else           { $source = $self->get_page($page)        }
+	die "No such page: $page\n" unless $source;
+	$$opts{callback}->($source) || die "Export Cancelled at page $page\n"
+		if $$opts{callback};
+
 	
-	my $source = $self->get_page($page);
 	return 0 unless $source->exists;
 	print STDERR "Exporting: $page\n";
 	
@@ -292,6 +368,67 @@ sub _export_page {
 	return 1;
 }
 
+sub _write_index {
+	my ($self, $exporter, $index) = @_;
+	
+	return if $self->get_page(':index')->exists;
+	print STDERR "Writing index\n";
+	my $page = $exporter->get_page(':index');
+	
+	$exporter->{_next} = $page;
+	$exporter->{_prev} = $page;
+	$page->set_parse_tree( ['Document', {},
+		['head1', {}, "Document index"],
+		['Para', {}, _conv_index($index) ],
+	] );
+	
+}
+
+sub _conv_index {
+	my $node = shift;
+	my $lvl = shift @$node;
+	my $nsp = shift @$node;
+	my @list;
+	my $prev = ':'; # no page can match this
+	for (@$node) {
+		if (ref $_) {
+			my $n = $$_[1];
+			unless ($n =~ /(^|:)$prev:*$/) { # show namespace in tree
+				$n =~ s/:+$//;
+				$n =~ s/^.*://;
+				push @list, ("\t"x$lvl)."* ",['bold', {}, $n], "\n";
+			}
+			push @list, _conv_index($_); # recurs
+		}
+		else { push @list, ("\t"x$lvl)."* ", ['link', {to => $nsp.$_}, $_], "\n" }
+		$prev = $_;
+	}
+	return @list;
+}
+
+=back
+
+=head2 Utility Methods
+
+These methods can be used by the interface but do not need to be reproduced
+in child handlers.
+
+=over 4
+
+=item C<get_selection(\%OPT, @PAGES..)>
+
+Returns a L<Zim::Selection> object for PAGES.
+Both PAGES and OPT are optional arguments.
+
+=cut
+
+sub get_selection {
+	my $self = shift;
+	my $opt = ref($_[0]) ? shift : {} ;
+	my @pages = @_ ? (@_) : (':') ;
+	return Zim::Selection->new($self, $opt, @pages);
+}
+
 =item C<add_child(NAMESPACE, CHILD, ...)>
 
 This will connect a new child object to this repository under a certain
@@ -319,14 +456,49 @@ sub add_child {
 	#warn "regex: $self->{regex}\n";
 }
 
-=back
+=item C<interwiki_lookup(KEY, PAGE)>
 
-=head2 Utility Methods
+=cut
 
-These methods can be used by the interface but do not need to be reproduced
-in child handlers.
+our %_urls;
 
-=over 4
+sub interwiki_lookup { # FIXME linked from Repository base for export stuff
+	my ($self, $key, $page) = @_;
+
+	my $url;
+	if (exists $_urls{$key}) { $url = $_urls{$key} }
+	else { # file lookup
+		# TODO repository specific urls here
+		
+		# Lookup named zim repositories
+		for (xdg_config_files('zim', 'repositories.list')) {
+			my ($hash) = Zim::File->new($_)->read_config;
+			next unless exists $$hash{$key};
+			my $file = Zim::File->abs_path($$hash{$key}, $ENV{HOME});
+			$url = Zim::File->path2uri($file);
+			$url =~ s#^file:#zim:#i;
+			$url .= '?{NAME}';
+			last;
+		}
+
+		# lookup interwiki config
+		unless (length $url) {
+			for (xdg_data_files(qw/zim urls.list/)) {
+				$url = Zim::File->new($_)->read_hash($key);
+				last if defined $url;
+			}
+		}
+	}
+	return unless length $url;
+	$_urls{$key} = $url;
+
+	unless ($url =~ s/{NAME}/$page/) {
+		$page =~ s/([^A-Za-z0-9\-_.!~*'()])/sprintf("%%%02X", ord($1))/ge;
+		$url =~ s/{URL}/$page/ or $url .= $page;
+	}
+	
+	return $url;
+}
 
 =item C<split_page(PAGE, LEVEL)>
 
@@ -447,6 +619,14 @@ sub _list_pages {
 	return @list;
 }
 
+sub _flush_cache {
+	my $self = shift;
+	for (grep /^:/, keys %$self) {
+		$self->{$_}->_flush_cache
+			if $self->{$_}->can('_flush_cache');
+	}
+}
+
 1;
 
 __END__
